;;; Projet

(define (evalprog program environnement)
  (define (eval-identifier identifier)
    (cdr (assq identifier environnement)))
  (define (eval-expression instruction)
    (let ((operation (car instruction))
	  (target (cadr instruction))
	  (source (caddr instruction)))
      (cons
       target
	(case operation
	  ('ld (eval-identifier source))
	  ('add (+ (eval-identifier target) (eval-identifier source)))
	  ('sub (- (eval-identifier target) (eval-identifier source)))
	  ('mult (* (eval-identifier target) (eval-identifier source)))
	  ('div (/ (eval-identifier target) (eval-identifier source)))))))
  (if (null? program)
      environnement
      (evalprog (cdr program)
		(cons (eval-expression (car program)) environnement))))


(evalprog '(
	    (ld R0 A)
	    (ld R1 B)
	    (sub R1 R0)
	    )
	  '(
	    (A . 1)
	    (B . 3)
	    ))

;;; Evaluation des expressions
(define (evalexpr expression environnement)
  (if (pair? expression)
      ((eval (cadr expression)) (evalexpr (car expression) environnement)
				(evalexpr (caddr expression) environnement))
      (cdr (assq expression environnement))))

(evalexpr '((A * C) + (B / C)) '((A . 1)(B . 2)(C . 3)))

(define (mkregister n)
  (string->symbol (string-append "R" (number->string n))))

(define (compile expression)
  (compile_aux expression 0))

(define (compile_aux expression nreg)
  (let ((liste-operation '((+ . add)(- . sub)(* . mult)(/ . div))))
    (cond ((pair? expression)
	   (append (append (compile_aux (car expression) nreg)
			   (compile_aux (caddr expression) (1+ nreg)))
		   (list (list
			  (cdr (assq (cadr expression) liste-operation))
			  (mkregister nreg) (mkregister (1+ nreg))))))
	  (else
	   (list (list 'ld (mkregister nreg) expression))))))

(compile '((A + B)*(C - D)))

(define add +)
(define sub -)
(define mult *)
(define div /)
(define ld define)
(define (exec program environnement)
  (eval (mexec program environnement)))

(define (mexec program environnement)
  (define (f environnement)
    (if (null? environnement)
	program
	(cons (list 'define (caar environnement) (cdar environnement))
	      (f (cdr environnement)))))
  (cons 'let (cons '() (f environnement))))

(exec '((ld R0 A)
        (ld R1 B)
        (add R0 R1))
       '((A . 12) (B . 24) (C . 36)))


